// custom-schema.ts
import { Schema, NodeSpec, MarkSpec } from "prosemirror-model";

// Define custom nodes
export const nodes = {
  text: {
    group: "inline",
  } as NodeSpec,
  star: {
    inline: true,
    group: "inline",
    toDOM() {
      return ["star", "⭐"];
    },
    parseDOM: [{ tag: "star" }],
  } as NodeSpec,
  paragraph: {
    group: "block",
    content: "inline*",
    toDOM() {
      return ["p", 0];
    },
    parseDOM: [{ tag: "p" }],
  } as NodeSpec,
  boring_paragraph: {
    group: "block",
    content: "text*",
    marks: "",
    toDOM() {
      return ["p", { class: "boring" }, 0];
    },
    parseDOM: [{ tag: "p", priority: 60 }],
  } as NodeSpec,
  blockquote: {
    content: "block+",
    group: "block",
    parseDOM: [{ tag: "blockquote" }],
    toDOM() {
      return ["blockquote", 0];
    },
  } as NodeSpec,
  horizontal_rule: {
    group: "block",
    selectable: false,
    parseDOM: [{ tag: "hr" }],
    toDOM() {
      return ["hr"];
    },
  } as NodeSpec,
  heading: {
    attrs: { level: { default: 1 } },
    content: "inline*",
    group: "block",
    defining: true,
    parseDOM: [
      { tag: "h1", attrs: { level: 1 } },
      { tag: "h2", attrs: { level: 2 } },
      { tag: "h3", attrs: { level: 3 } },
    ],
    toDOM(node) {
      return ["h" + node.attrs.level, 0];
    },
  } as NodeSpec,
  code_block: {
    content: "text*",
    group: "block",
    marks: "",
    defining: true,
    code: true,
    parseDOM: [{ tag: "pre", preserveWhitespace: "full" }],
    toDOM() {
      return ["pre", ["code", 0]];
    },
  } as NodeSpec,
  image: {
    inline: true,
    attrs: {
      src: {},
      alt: { default: null },
      title: { default: null },
    },
    group: "inline",
    draggable: true,
    parseDOM: [
      {
        tag: "img[src]",
        getAttrs(dom: any) {
          return {
            src: dom.getAttribute("src"),
            alt: dom.getAttribute("alt"),
            title: dom.getAttribute("title"),
          };
        },
      },
    ],
    toDOM(node) {
      return ["img", node.attrs];
    },
  } as NodeSpec,
  doc: {
    content: "block+",
  } as NodeSpec,
};

// Define custom marks
export const marks = {
  shouting: {
    toDOM() {
      return ["shouting", 0];
    },
    parseDOM: [{ tag: "shouting" }],
  } as MarkSpec,
  link: {
    attrs: { href: {} },
    toDOM(node) {
      return ["a", { href: node.attrs.href }, 0];
    },
    parseDOM: [
      {
        tag: "a",
        getAttrs(dom) {
          return { href: dom.getAttribute("href") };
        },
      },
    ],
    inclusive: false,
  } as MarkSpec,
  bold: {
    parseDOM: [
      { tag: "strong" },
      { tag: "b", getAttrs: () => null },
      {
        style: "font-weight",
        getAttrs: (value: string) => (value === "bold" ? null : false),
      },
    ],
    toDOM() {
      return ["strong", 0];
    },
  } as MarkSpec,
  emphasis: {
    parseDOM: [
      { tag: "em" },
      { tag: "i", getAttrs: () => null },
      {
        style: "font-style",
        getAttrs: (value: string) => (value === "italic" ? null : false),
      },
    ],
    toDOM() {
      return ["em", 0];
    },
  } as MarkSpec,
};

// Export the schema
export const customSchema = new Schema({ nodes, marks });
