import { LitElement, css, html } from 'lit';
import { property, customElement } from 'lit/decorators.js';
import { resolveRouterPath } from '../router';
import { Relay } from 'nostr-tools';
import { WindowNostr } from 'nostr-tools/nip07';


import '@shoelace-style/shoelace/dist/components/card/card.js';
import '@shoelace-style/shoelace/dist/components/button/button.js';

import { styles } from '../styles/shared-styles';


declare global {
  interface Window {
    nostr?: WindowNostr;
  }
}

@customElement('app-home')
export class AppHome extends LitElement {

  // For more information on using properties and state in lit
  // check out this link https://lit.dev/docs/components/properties/
  @property() message = 'Welcome to my demo!';
  @property({ type: String }) nostrAddy = '';
  @property({ type: String }) bio = '';
  @property({ type: String }) profilePic = '/assets/img/default_pfp.png';
  @property({ type: Boolean }) isSignedIn = false;
  @property({ type: String }) publicKey = '';


  static styles = [
    styles,
    css`

    /*
========================================
Profile Picture Container
========================================
*/
.profile-picture-container {
  display: grid;
  align-itens: center;
  grid-template-columns: 350px 150px;
  grid-template-rows: 250px;
  grid-column-gap: 10px;

}


.profile-pic {
  grid-area: 1/1;
  margin-top: 30px;
  margin-left: 100px;
  margin-bottom: 5px;

  border-radius: 50%; /* this makes the profile picture round */
}

.personal-msg {
  grid-area: 1/2;
  align-content: center;
  align-items: center;
}



.nav-button-container {
  justify-self: center;
  align-self: end;
}

  `];

  async firstUpdated() {
    // this method is a lifecycle event in lit
    // for more info check out the lit docs https://lit.dev/docs/components/lifecycle/
    console.log('Welcome to my home page');
  }


  connectedCallback(): void {
    super.connectedCallback();

    this.addEventListener('user-state-changed', (event: Event) => {
    const detail = (event as CustomEvent).detail;
    this.nostrAddy = detail.nostrAddy;
    this.bio = detail.bio;
    this.profilePic = detail.profilePic;
    this.isSignedIn = detail.isSignedIn;
  });
  }



  disconnectedCallback(): void {
    super.disconnectedCallback();
    window.removeEventListener('profile-updated', this.updateProfileFromEvent.bind(this));
    window.removeEventListener('user-signed-out', this.handlesSignOut.bind(this));
  }

  updateProfileFromEvent(event: Event) {
    console.log('Profile updated event received:', (event as CustomEvent).detail);

    const detail = (event as CustomEvent).detail;
    this.nostrAddy = detail.nostrAddy;
    this.bio = detail.bio;
    this.profilePic = detail.profilePic;
    this.isSignedIn = true;
    this.requestUpdate();
  }

  handlesSignOut() {
    this.profilePic = '/dist/assets/img/default_pfp.png';
    this.bio = '';
    this.nostrAddy = 'Guest';
    this.isSignedIn = false;
    this.requestUpdate();
  }

  async fetchAndDisplayProfile(pubkey: string) {
    this.nostrAddy = pubkey;
    this.bio = 'Loading profile info...';

    try {
      // try to fetch profile metadata from a relay
      const relay = await Relay.connect('wss://notes.miguelalmodo.com'); // Example URL
      const sub = relay.subscribe([{ kinds: [0], authors: [pubkey] }], {
        onevent: (event) => {
          const profileData = JSON.parse(event.content);
          this.nostrAddy = profileData.nip05 || 'No address available';
          this.bio = profileData.about || 'No bio available';
          this.profilePic = profileData.picture || '/dist/assets/img/default_pfp.png';
          this.requestUpdate();
        },
        oneose: () => sub.close(),
      });
    } catch (error) {
      console.error('Error fetching profile:', error);
      this.displayGuestView();
    }
  }


  displayGuestView() {
    // Set initial values for guest view
    this.nostrAddy = '';
    this.bio = 'Welcome, guest! Please sign in with a browser extension to view your profile.';
    this.profilePic = '/dist/assets/img/default_pfp.png';
    this.isSignedIn = false;
  }


  share() {
    if ((navigator as any).share) {
      (navigator as any).share({
        title: 'A MiggyMofongo Project',
        text: 'This is a personal progressive social web app',
        url: 'https://miguelalmodo.com/dist',
      });
    }
  }




  render() {
    return html`
      <app-header></app-header>

      <main>
        <div id="welcomeBar">
          <sl-card id="welcomeCard">
            <div slot="header">
              <h2>Welcome to  ${this.nostrAddy || 'Guest'}'s Profile</h2>
            </div>
            <p>
              You can upgrade your personal website with a social
              protocol to do things like browse a feed, compose
              a note, or post to your network.
            </p>


            <div class="profile-picture-container">
<img class="profile-pic" src="${this.profilePic}" alt="Profile Picture" width="200px" height="200px">

              <p class="personal-msg"><b>${this.bio || 'Welcome, guest! Please sign in to view your profile.'}</b></p>

            </div>
<div class="nav-button-container">
          <sl-button href="${resolveRouterPath('about')}" variant="primary">About</sl-button>
          <sl-button href="${resolveRouterPath('note-wall')}" variant="primary">Feed</sl-button>
          <sl-button href="${resolveRouterPath('write')}" variant="primary">Compose</sl-button>
</div>
            ${'share' in navigator
              ? html`<sl-button slot="footer" variant="default" @click="${this.share}">
                        <sl-icon slot="prefix" name="share"></sl-icon>
                        Share with a homie!
                      </sl-button>`
              : null}
          </sl-card>



         </div>
      </main>
    `;
  }




}
