import { LitElement, html, css } from 'lit';
import { customElement, query } from 'lit/decorators.js';

import '@shoelace-style/shoelace/dist/components/card/card.js';

import '@shoelace-style/shoelace/dist/components/button/button.js';

import { styles } from '../../styles/shared-styles';
import { editorStyles } from './write-styles';


import {EditorState} from 'prosemirror-state'
import { Transaction } from 'prosemirror-state';
import { EditorView } from 'prosemirror-view';
import { customSchema } from '../../components/schema';
import { keymap } from 'prosemirror-keymap';
import { toggleMark } from 'prosemirror-commands';
import {undo, redo, history} from 'prosemirror-history'
import { baseKeymap } from 'prosemirror-commands';


// function for error reports

  function errorReport(message: string): void {
	console.error(message);
  }

/* COMMANDS
commands are functions that take an editor state
and a dispatch function and returns a boolean
to implement editing actions.*/


  /**this command inserts a star by your cursor. it sets
   * type to the star node, and creates one when
   * dispatch is provided  */

function insertStar(state: EditorState, dispatch?: (tr: Transaction) => void): boolean {
	const star = customSchema.nodes.star;
	const { $from } = state.selection;

	// If the parent cannot replace with the 'star' node, return false
	if (!$from.parent.canReplaceWith($from.index(), $from.index(), star)) {
	  return false;
	}

	// If dispatch is provided, apply the transaction
	if (dispatch) {
	  dispatch(state.tr.replaceSelectionWith(star.create()));
	}

	// Always return true if insertion conditions are met
	return true;
  }

   /* function that inserts an <hr> line break*/

function insertHR(state: EditorState, dispatch?: (tr: Transaction) => void): boolean {
  const hr = customSchema.nodes.horizontal_rule; // attach the HR node from the schema to a variable
  if (!hr) {
    console.error('HR node is not defined in the schema.');
    return false;
  }

  if (dispatch) {
    dispatch(state.tr.replaceSelectionWith(hr.create()));
  }
  return true;
}

/**command function that inserts a blockquote */

function insertQuote(state: EditorState, dispatch?: (tr: Transaction) => void): boolean {
    console.log("you just put a blockquote");

    const { schema, tr, selection } = state;
    const blockquote = schema.nodes.blockquote;

    // Handle when no selection is made (empty selection)
    if (selection.empty) {
        console.log("No selection, inserting a blockquote...");

        const blockquoteNode = blockquote.createAndFill();
        if (blockquoteNode) {
            // Only dispatch if `dispatch` is defined
            if (dispatch) {
                dispatch(tr.replaceSelectionWith(blockquoteNode));
            }
        }
    } else {
        console.log("Inserting blockquote at selection");

        // Handle the case where there is a selection
        const blockquoteNode = blockquote.createAndFill();
        if (blockquoteNode) {
            // Only dispatch if `dispatch` is defined
            if (dispatch) {
                dispatch(tr.replaceSelectionWith(blockquoteNode));
            }
        }
    }

    return true;
}


/* this command will apply the link mark to hyperlink your selection  */

function toggleLink(state: EditorState, dispatch?: (tr: Transaction) => void): boolean {
	let {doc, selection} = state;
	if (selection.empty) return false;
	let attrs = null
	if (!doc.rangeHasMark(selection.from, selection.to, customSchema.marks.link)) {
	  attrs = {href: prompt("Paste your URL in here please", "")}
	  if (!attrs.href) return false
	}
	return toggleMark(customSchema.marks.link, attrs)(state, dispatch)
  }

  /* these functions add the bold and emphasis marks  */
  function toggleBold(state: EditorState, dispatch?: (tr: Transaction) => void): boolean {
	return toggleMark(customSchema.marks.bold)(state, dispatch);
  }

  function toggleEmphasis(state: EditorState, dispatch?: (tr: Transaction) => void): boolean {
	return toggleMark(customSchema.marks.emphasis)(state, dispatch)
  }


  // custom keymap to apply to state
const customKeymap = keymap({
	'Shift-Space': insertStar,
	'Ctrl-g': (state, dispatch) => {
    console.log("Ctrl-b pressed, toggling shouting mark...");
    return toggleMark(customSchema.marks.shouting)(state, dispatch);
  },
  	'Ctrl-k': (state, dispatch) => {
		console.log("you should have just gotten an alert to place a url into a hyperlink");
		return toggleLink(state, dispatch);
	},
	'Ctrl-e': toggleEmphasis,
	'Ctrl-b': toggleBold,

	'Ctrl-h': insertHR,
	'Ctrl-q': insertQuote
	},


  );

@customElement('app-write')
export class AppWrite extends LitElement {

	// references the class name of the container that prosemirror's editor
	// will render to
	@query('.ProseMirror') editorContainer!: HTMLElement;

	private editorView!: EditorView

    static styles = [
        styles,

		editorStyles,

		css`:root {
  --main-line-color: hsl(234, 62%, 86%);
  --side-line-color: hsl(350, 100%, 91%);
  --paper-color: hsl(0, 15%, 95%);
  --ink-color: hsl(0, 0%, 12%);
  --line-thickness: 3px;
  --top-space: 4lh;
}`
    	];

	constructor() {
			super();
		}

	destroy() {
		this.editorView.destroy
		}


	connectedCallback(): void {

	super.connectedCallback();
    console.log('AppWrite added to the DOM')
	   }

	disconnectedCallback(): void {
		super.disconnectedCallback();
		if (this.editorView) {
	  		this.editorView.destroy();
		}
   	}



	protected async firstUpdated() {
		console.log("Welcome to the compose page");
		await this.updateComplete;

		this.initializeEditor()
		if (!this.editorContainer) {
			errorReport('Editor container not here');
			return;
		}
	}

	private initializeEditor() {

			if (!this.editorContainer) {
			errorReport('Editor container not here');
				return;
			}

		/*  a prepopulated document of initial content to guide the user */


			let doc = customSchema.node('doc', null, [
				customSchema.node('heading', null, [customSchema.text('Título h1')]),

				customSchema.node('blockquote', null, [
					customSchema.node('paragraph', null, [customSchema.text('"WEPA"')]),
			  	]),
				customSchema.node('paragraph', null, [customSchema.text('Escribe algo')]),
				customSchema.node('boring_paragraph', null, [
			  	customSchema.text('no se puede marcar ningún texto en párrafos aburridos como este. mira el <hr> debajo de este párafo. puedes colocar uno con Ctrl + h'),
				]),
				customSchema.node('horizontal_rule', null),
				customSchema.node('paragraph', null, [customSchema.text('Escribe mas aquí...')]),
				customSchema.node('paragraph', null, [customSchema.text('Presione Shift + Space para colocar una estrella')]),
				customSchema.node('paragraph', null, [customSchema.text('Presione Ctrl + g para marcar la selección con la marca "shouting" ')]),
				customSchema.node('heading', { level: 2 }, [customSchema.text('Título h2')]),

			])


			if (!doc) {
			console.error("failed to create initial document")
			return;
			}



			/* here i create a state with doc to,
			which places the prepopulated document of content
			and include history and keymap as plugins
			 */

			const state = EditorState.create({
    			doc: doc,
    			plugins: [
					history(),
      				keymap({
        			'Mod-z': undo,
        			'Mod-y': redo,
      				}),
				  	customKeymap,
      				keymap(baseKeymap),

    				],
	 			});


					/* this  */

			this.editorView = new EditorView(this.editorContainer, {
				state,
				dispatchTransaction: (transaction) => {
					console.log('Document size went from', transaction.before.content.size, "to",
						transaction.doc.content.size
					)
				let newState = this.editorView.state.apply(transaction);
				this.editorView.updateState(newState);
				}
				});
				console.log(state)

			}

    protected render() {
        return html`
        <app-header ?enableBack="${true}"></app-header>

<main>
	<div class="ProseMirror">	Try selecting some text below to test out a minimal rich text editor.

	</div>
</main>

        `;
	}
}